
package org.mule.module.hbase.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.module.hbase.processors.CheckAndDeleteValueMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.w3c.dom.Element;

public class CheckAndDeleteValueDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public CheckAndDeleteValueDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(CheckAndDeleteValueMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("tableName")!= null)&&(!StringUtils.isBlank(element.getAttribute("tableName")))) {
            builder.addPropertyValue("tableName", element.getAttribute("tableName"));
        }
        if ((element.getAttribute("rowKey")!= null)&&(!StringUtils.isBlank(element.getAttribute("rowKey")))) {
            builder.addPropertyValue("rowKey", element.getAttribute("rowKey"));
        }
        if ((element.getAttribute("checkColumnFamilyName")!= null)&&(!StringUtils.isBlank(element.getAttribute("checkColumnFamilyName")))) {
            builder.addPropertyValue("checkColumnFamilyName", element.getAttribute("checkColumnFamilyName"));
        }
        if ((element.getAttribute("checkColumnQualifier")!= null)&&(!StringUtils.isBlank(element.getAttribute("checkColumnQualifier")))) {
            builder.addPropertyValue("checkColumnQualifier", element.getAttribute("checkColumnQualifier"));
        }
        if ((element.getAttribute("checkValue-ref")!= null)&&(!StringUtils.isBlank(element.getAttribute("checkValue-ref")))) {
            if (element.getAttribute("checkValue-ref").startsWith("#")) {
                builder.addPropertyValue("checkValue", element.getAttribute("checkValue-ref"));
            } else {
                builder.addPropertyValue("checkValue", (("#[registry:"+ element.getAttribute("checkValue-ref"))+"]"));
            }
        }
        if ((element.getAttribute("deleteColumnFamilyName")!= null)&&(!StringUtils.isBlank(element.getAttribute("deleteColumnFamilyName")))) {
            builder.addPropertyValue("deleteColumnFamilyName", element.getAttribute("deleteColumnFamilyName"));
        }
        if ((element.getAttribute("deleteColumnQualifier")!= null)&&(!StringUtils.isBlank(element.getAttribute("deleteColumnQualifier")))) {
            builder.addPropertyValue("deleteColumnQualifier", element.getAttribute("deleteColumnQualifier"));
        }
        if ((element.getAttribute("deleteTimestamp")!= null)&&(!StringUtils.isBlank(element.getAttribute("deleteTimestamp")))) {
            builder.addPropertyValue("deleteTimestamp", element.getAttribute("deleteTimestamp"));
        }
        if ((element.getAttribute("deleteAllVersions")!= null)&&(!StringUtils.isBlank(element.getAttribute("deleteAllVersions")))) {
            builder.addPropertyValue("deleteAllVersions", element.getAttribute("deleteAllVersions"));
        }
        if ((element.getAttribute("lock-ref")!= null)&&(!StringUtils.isBlank(element.getAttribute("lock-ref")))) {
            if (element.getAttribute("lock-ref").startsWith("#")) {
                builder.addPropertyValue("lock", element.getAttribute("lock-ref"));
            } else {
                builder.addPropertyValue("lock", (("#[registry:"+ element.getAttribute("lock-ref"))+"]"));
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
