
package org.mule.module.hbase.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.api.lifecycle.Disposable;
import org.mule.api.lifecycle.Initialisable;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.parsers.generic.AutoIdUtils;
import org.mule.module.hbase.adapters.HbaseCloudConnectorLifecycleAdapter;
import org.mule.util.TemplateParser;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class HbaseCloudConnectorConfigDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public HbaseCloudConnectorConfigDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        String name = element.getAttribute("name");
        if ((name == null)||StringUtils.isBlank(name)) {
            element.setAttribute("name", AutoIdUtils.getUniqueName(element, "mule-bean"));
        }
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(HbaseCloudConnectorLifecycleAdapter.class.getName());
        if (Initialisable.class.isAssignableFrom(HbaseCloudConnectorLifecycleAdapter.class)) {
            builder.setInitMethodName(Initialisable.PHASE_NAME);
        }
        if (Disposable.class.isAssignableFrom(HbaseCloudConnectorLifecycleAdapter.class)) {
            builder.setDestroyMethodName(Disposable.PHASE_NAME);
        }
        if ((element.getAttribute("facade-ref")!= null)&&(!StringUtils.isBlank(element.getAttribute("facade-ref")))) {
            builder.addPropertyValue("facade", new RuntimeBeanReference(element.getAttribute("facade-ref")));
        }
        Element propertiesListElement = null;
        propertiesListElement = DomUtils.getChildElementByTagName(element, "properties");
        List<Element> propertiesMapChilds = null;
        if (propertiesListElement!= null) {
            String propertiesRef = propertiesListElement.getAttribute("ref");
            if ((propertiesRef!= null)&&(!StringUtils.isBlank(propertiesRef))) {
                if ((!propertiesRef.startsWith(patternInfo.getPrefix()))&&(!propertiesRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("properties", new RuntimeBeanReference(propertiesRef));
                } else {
                    builder.addPropertyValue("properties", propertiesRef);
                }
            } else {
                ManagedMap properties = new ManagedMap();
                propertiesMapChilds = DomUtils.getChildElementsByTagName(propertiesListElement, "property");
                if (propertiesMapChilds!= null) {
                    if (propertiesMapChilds.size() == 0) {
                        propertiesMapChilds = DomUtils.getChildElements(propertiesListElement);
                    }
                    for (Element propertiesChild: propertiesMapChilds) {
                        String propertiesValueRef = propertiesChild.getAttribute("value-ref");
                        String propertiesKeyRef = propertiesChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((propertiesValueRef!= null)&&(!StringUtils.isBlank(propertiesValueRef))) {
                            if ((!propertiesValueRef.startsWith(patternInfo.getPrefix()))&&(!propertiesValueRef.endsWith(patternInfo.getSuffix()))) {
                                valueObject = new RuntimeBeanReference(propertiesValueRef);
                            } else {
                                valueObject = propertiesValueRef;
                            }
                        } else {
                            valueObject = propertiesChild.getTextContent();
                        }
                        if ((propertiesKeyRef!= null)&&(!StringUtils.isBlank(propertiesKeyRef))) {
                            keyObject = new RuntimeBeanReference(propertiesKeyRef);
                        } else {
                            keyObject = propertiesChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = propertiesChild.getTagName();
                        }
                        properties.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("properties", properties);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        return definition;
    }

}
