
package org.mule.module.hbase.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.module.hbase.processors.ModifyColumnFamilyMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class ModifyColumnFamilyDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public ModifyColumnFamilyDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(ModifyColumnFamilyMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("tableName")!= null)&&(!StringUtils.isBlank(element.getAttribute("tableName")))) {
            builder.addPropertyValue("tableName", element.getAttribute("tableName"));
        }
        if ((element.getAttribute("columnFamilyName")!= null)&&(!StringUtils.isBlank(element.getAttribute("columnFamilyName")))) {
            builder.addPropertyValue("columnFamilyName", element.getAttribute("columnFamilyName"));
        }
        if ((element.getAttribute("maxVersions")!= null)&&(!StringUtils.isBlank(element.getAttribute("maxVersions")))) {
            builder.addPropertyValue("maxVersions", element.getAttribute("maxVersions"));
        }
        if ((element.getAttribute("blocksize")!= null)&&(!StringUtils.isBlank(element.getAttribute("blocksize")))) {
            builder.addPropertyValue("blocksize", element.getAttribute("blocksize"));
        }
        if (element.hasAttribute("compressionType")) {
            builder.addPropertyValue("compressionType", element.getAttribute("compressionType"));
        }
        if (element.hasAttribute("compactionCompressionType")) {
            builder.addPropertyValue("compactionCompressionType", element.getAttribute("compactionCompressionType"));
        }
        if ((element.getAttribute("inMemory")!= null)&&(!StringUtils.isBlank(element.getAttribute("inMemory")))) {
            builder.addPropertyValue("inMemory", element.getAttribute("inMemory"));
        }
        if ((element.getAttribute("timeToLive")!= null)&&(!StringUtils.isBlank(element.getAttribute("timeToLive")))) {
            builder.addPropertyValue("timeToLive", element.getAttribute("timeToLive"));
        }
        if ((element.getAttribute("blockCacheEnabled")!= null)&&(!StringUtils.isBlank(element.getAttribute("blockCacheEnabled")))) {
            builder.addPropertyValue("blockCacheEnabled", element.getAttribute("blockCacheEnabled"));
        }
        if (element.hasAttribute("bloomFilterType")) {
            builder.addPropertyValue("bloomFilterType", element.getAttribute("bloomFilterType"));
        }
        if ((element.getAttribute("replicationScope")!= null)&&(!StringUtils.isBlank(element.getAttribute("replicationScope")))) {
            builder.addPropertyValue("replicationScope", element.getAttribute("replicationScope"));
        }
        Element valuesListElement = null;
        valuesListElement = DomUtils.getChildElementByTagName(element, "values");
        List<Element> valuesMapChilds = null;
        if (valuesListElement!= null) {
            String valuesRef = valuesListElement.getAttribute("ref");
            if ((valuesRef!= null)&&(!StringUtils.isBlank(valuesRef))) {
                if ((!valuesRef.startsWith(patternInfo.getPrefix()))&&(!valuesRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("values", new RuntimeBeanReference(valuesRef));
                } else {
                    builder.addPropertyValue("values", valuesRef);
                }
            } else {
                ManagedMap values = new ManagedMap();
                valuesMapChilds = DomUtils.getChildElementsByTagName(valuesListElement, "value");
                if (valuesMapChilds!= null) {
                    if (valuesMapChilds.size() == 0) {
                        valuesMapChilds = DomUtils.getChildElements(valuesListElement);
                    }
                    for (Element valuesChild: valuesMapChilds) {
                        String valuesValueRef = valuesChild.getAttribute("value-ref");
                        String valuesKeyRef = valuesChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((valuesValueRef!= null)&&(!StringUtils.isBlank(valuesValueRef))) {
                            if ((!valuesValueRef.startsWith(patternInfo.getPrefix()))&&(!valuesValueRef.endsWith(patternInfo.getSuffix()))) {
                                valueObject = new RuntimeBeanReference(valuesValueRef);
                            } else {
                                valueObject = valuesValueRef;
                            }
                        } else {
                            valueObject = valuesChild.getTextContent();
                        }
                        if ((valuesKeyRef!= null)&&(!StringUtils.isBlank(valuesKeyRef))) {
                            keyObject = new RuntimeBeanReference(valuesKeyRef);
                        } else {
                            keyObject = valuesChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = valuesChild.getTagName();
                        }
                        values.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("values", values);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
