
package org.mule.module.hbase.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.module.hbase.processors.ScanTableMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.w3c.dom.Element;

public class ScanTableDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public ScanTableDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(ScanTableMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("tableName")!= null)&&(!StringUtils.isBlank(element.getAttribute("tableName")))) {
            builder.addPropertyValue("tableName", element.getAttribute("tableName"));
        }
        if ((element.getAttribute("columnFamilyName")!= null)&&(!StringUtils.isBlank(element.getAttribute("columnFamilyName")))) {
            builder.addPropertyValue("columnFamilyName", element.getAttribute("columnFamilyName"));
        }
        if ((element.getAttribute("columnQualifier")!= null)&&(!StringUtils.isBlank(element.getAttribute("columnQualifier")))) {
            builder.addPropertyValue("columnQualifier", element.getAttribute("columnQualifier"));
        }
        if ((element.getAttribute("timestamp")!= null)&&(!StringUtils.isBlank(element.getAttribute("timestamp")))) {
            builder.addPropertyValue("timestamp", element.getAttribute("timestamp"));
        }
        if ((element.getAttribute("maxTimestamp")!= null)&&(!StringUtils.isBlank(element.getAttribute("maxTimestamp")))) {
            builder.addPropertyValue("maxTimestamp", element.getAttribute("maxTimestamp"));
        }
        if ((element.getAttribute("caching")!= null)&&(!StringUtils.isBlank(element.getAttribute("caching")))) {
            builder.addPropertyValue("caching", element.getAttribute("caching"));
        }
        if ((element.getAttribute("cacheBlocks")!= null)&&(!StringUtils.isBlank(element.getAttribute("cacheBlocks")))) {
            builder.addPropertyValue("cacheBlocks", element.getAttribute("cacheBlocks"));
        }
        if ((element.getAttribute("maxVersions")!= null)&&(!StringUtils.isBlank(element.getAttribute("maxVersions")))) {
            builder.addPropertyValue("maxVersions", element.getAttribute("maxVersions"));
        }
        if ((element.getAttribute("startRowKey")!= null)&&(!StringUtils.isBlank(element.getAttribute("startRowKey")))) {
            builder.addPropertyValue("startRowKey", element.getAttribute("startRowKey"));
        }
        if ((element.getAttribute("stopRowKey")!= null)&&(!StringUtils.isBlank(element.getAttribute("stopRowKey")))) {
            builder.addPropertyValue("stopRowKey", element.getAttribute("stopRowKey"));
        }
        if ((element.getAttribute("fetchSize")!= null)&&(!StringUtils.isBlank(element.getAttribute("fetchSize")))) {
            builder.addPropertyValue("fetchSize", element.getAttribute("fetchSize"));
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
